/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.FlowVelocityData;
import org.dive4elements.river.artifacts.model.WQKms;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.model.FlowVelocityMeasurementValue.FastFlowVelocityMeasurementValue;
import org.dive4elements.river.themes.ThemeDocument;
import org.jfree.data.xy.XYSeries;

/**
 * Add data to chart/generator.
 *
 * @author <a href="mailto:bjoern.ricks@intevation.de">Björn Ricks</a>
 */
public class QOutProcessor extends DefaultProcessor {

    public static final String I18N_LONGITUDINAL_LABEL = "common.export.csv.header.q";

    public static final String I18N_LONGITUDINAL_LABEL_DEFAULT = "Q [m\u00b3/s]";

    /** Private log. */
    private static final Logger log = Logger.getLogger(QOutProcessor.class);

    @Override
    public void doOut(final DiagramGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible) {
        final CallContext context = generator.getContext();
        final Object data = bundle.getData(context);
        final XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);
        final String facetName = bundle.getFacetName();

        if (facetName.equals(FacetTypes.FLOW_VELOCITY_DISCHARGE)) {
            if (data instanceof FlowVelocityData) {
                final FlowVelocityData fData = (FlowVelocityData) data;
                StyledSeriesBuilder.addPoints(series, fData.getQPoints(), true);
            } else {
                final FastFlowVelocityMeasurementValue fData = (FastFlowVelocityMeasurementValue) data;
                final double[][] points = new double[][] { { fData.getStation() }, { fData.getQ() } };
                StyledSeriesBuilder.addPoints(series, points, true);
            }
        } else {
            final WQKms wqkms = (WQKms) data;
            StyledSeriesBuilder.addStepPointsKmQ(series, wqkms);
        }

        generator.addAxisSeries(series, this.axisName, visible);
    }

    @Override
    public void doOut(final XYChartGenerator generator, final ArtifactAndFacet bundle, final ThemeDocument theme, final boolean visible, final int index) {
        final CallContext context = generator.getContext();
        final WQKms wqkms = (WQKms) bundle.getData(context);

        final XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);

        StyledSeriesBuilder.addStepPointsKmQ(series, wqkms);

        generator.addAxisSeries(series, index, visible);

        /* Check if the diagram should be inverted */
        generator.setInverted(wqkms.guessRTLData());
    }

    @Override
    public String getAxisLabel(final DiagramGenerator generator) {
        return generator.msg(I18N_LONGITUDINAL_LABEL, I18N_LONGITUDINAL_LABEL_DEFAULT);
    }

    /**
     * Returns true if facettype is q-type.
     */
    @Override
    public boolean canHandle(final String facetType) {
        if (facetType == null) {
            return false;
        }

        if (facetType.equals(FacetTypes.STATIC_WQKMS_Q) || facetType.equals(FacetTypes.LONGITUDINAL_Q)
                || facetType.startsWith(FacetTypes.DISCHARGE_LONGITUDINAL_Q) || facetType.startsWith(FacetTypes.FLOW_VELOCITY_DISCHARGE)) {
            return true;
        }
        return false;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
